import {
  UserActionType,
  GetAllUsersActionSuccess,
  GetAllUsersActionError,
  UserActionUnion,
  SearchUserActionSuccess,
  SearchUserActionError,
  CreareFormUserEditActionSuccess,
  CreareFormUserEditActionActionError,
  DeleteUserActionSuccess,
  DeleteUserActionError,
  CreateFormUserAddActionSuccess,
  CreateFormUserAddActionError,
  SaveUserFormActionSuccess,
  SaveUserFormActionError,
  UpdateUserFormAction,
  UpdateUserFormActionError,
  UpdateUserFormActionSuccess,
} from './users.actions';
import { map, mergeMap, Observable, catchError, of } from 'rxjs';
import { UsersService } from './../../shared/services/users.service';
import { Injectable } from '@angular/core';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { User } from 'src/app/shared/models/user.model';

@Injectable()
export class UserEffects {
  constructor(
    private usersService: UsersService,
    private effectActions: Actions
  ) {}

  /** creation of effects about all cations related to a user */

  //create getAllUsers effect

  getAllUsersEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionType.GET_ALL_USERS),
      mergeMap(() => {
        return this.usersService.getAllUsers().pipe(
          map((users: User[]) => new GetAllUsersActionSuccess(users)),
          catchError((err) => of(new GetAllUsersActionError(err.message)))
        );
      })
    )
  );

  //create searchUser effect

  searchUserEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionType.SEARCH_A_USER),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.searchUser(userAction.payload).pipe(
          map((users: User[]) => new SearchUserActionSuccess(users)),
          catchError((err) => of(new SearchUserActionError(err.message)))
        );
      })
    )
  );

  //create a user add form effect
  createAddUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionType.CREATE_FORM_ADD_USER),
      map(() => {
        return new CreateFormUserAddActionSuccess({});
      }),
      catchError((err) => of(new CreateFormUserAddActionError(err.message)))
    )
  );

  //saving user form data effect

  saveUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionType.SAVE_USERFORM_ADD),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.createUser(userAction.payload).pipe(
          map((user) => new SaveUserFormActionSuccess(user)),
          catchError((err) => of(new SaveUserFormActionError(err.message)))
        );
      })
    )
  );

  // create an update user form effect and fill it with data of a user to update

  createUpdateUserFormEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionType.CREATE_FORM_USER_EDIT),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.getUserById(userAction.payload.id).pipe(
          map((user: User) => new CreareFormUserEditActionSuccess(user)),
          catchError((err) =>
            of(new CreareFormUserEditActionActionError(err.message))
          )
        );
      })
    )
  );

  // update a user with data of form effect

  updateUserFormDataEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionType.UPDATE_USER_FORM),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.updateUser(userAction.payload).pipe(
          map((user: User) => new UpdateUserFormActionSuccess(user)),
          catchError((err) => of(new UpdateUserFormActionError(err.message)))
        );
      })
    )
  );

  //deleteUser effect

  deleteUserEffect: Observable<UserActionUnion> = createEffect(() =>
    this.effectActions.pipe(
      ofType(UserActionType.DELETE_A_USER),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.deleteUser(userAction.payload.id).pipe(
          map(() => new DeleteUserActionSuccess(userAction.payload)),
          catchError((err) => of(new DeleteUserActionError(err.message)))
        );
      })
    )
  );
}
