import { Action } from '@ngrx/store';
import { User } from 'src/app/shared/models/user.model';
import { MsgState } from '../msg.state';
import { UserState } from './user.state';
import { UserActionUnion, UserActionType } from './users.actions';

/** create reducer,
 * a reducer need a initial state and an action to perform*/

const initialUserState: UserState = {
  usersList: [],
  currentUser: null,
  msgState: MsgState.INIT_STATE,
  userServerErrorMsg: 'No error message',
};

export function userReducer(
  state: UserState = initialUserState,
  action: Action
): UserState {

  switch (action.type) {

    // get all users actions
    case UserActionType.GET_ALL_USERS:
      return { ...state, msgState: MsgState.LOADING };

    case UserActionType.GET_ALL_USERS_SUCCESS:
      return {
        ...state,
        msgState: MsgState.LOADED,
        usersList: (<UserActionUnion>action).payload,
      };

    case UserActionType.GET_ALL_USERS_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    // search a user actions
    case UserActionType.SEARCH_A_USER:
      return { ...state, msgState: MsgState.LOADING };

    case UserActionType.SEARCH_A_USER_SUCCESS:
      return {
        ...state,
        msgState: MsgState.LOADED,
        usersList: (<UserActionUnion>action).payload,
      };

    case UserActionType.SEARCH_A_USER_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    // create a user form actions
    case UserActionType.CREATE_FORM_ADD_USER:
      return {
        ...state,
        msgState: MsgState.FORM_LOADING,
      };

    case UserActionType.CREATE_FORM_ADD_USER_SUCCESS:
      return {
        ...state,
        msgState: MsgState.FORM_LOADED,
      };

    case UserActionType.CREATE_FORM_ADD_USER_ERROR:
      return {
        ...state,
        msgState: MsgState.FORM_LOAD_ERR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    // save a user (data of form) action
    case UserActionType.SAVE_USERFORM_ADD:
      return {
        ...state,
        msgState: MsgState.DEFAULT_MSG,
      };
    case UserActionType.SAVE_USERFORM_ADD_SUCCESS:
      let createdUser: User = (<UserActionUnion>action).payload;
      let usersListCopy1: User[] = [...state.usersList];
      usersListCopy1.push(createdUser);

      return {
        ...state,
        msgState: MsgState.LOADED,
        usersList: usersListCopy1,
      };

    case UserActionType.SAVE_USERFORM_ADD_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    // create a edit user form actions
    case UserActionType.CREATE_FORM_USER_EDIT:
      return {
        ...state,
        msgState: MsgState.FORM_LOADING,
      };

    case UserActionType.CREATE_FORM_USER_EDIT_SUCCESS:
      let modifiedUser: User = (<UserActionUnion>action).payload;
      return {
        ...state,
        msgState: MsgState.FORM_LOADED,
        currentUser: modifiedUser,
      };

    case UserActionType.CREATE_FORM_USER_EDIT_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    // update user with user form data

    case UserActionType.UPDATE_USER_FORM:
      return {
        ...state,
        msgState: MsgState.DEFAULT_MSG,
      };

    case UserActionType.UPDATE_USER_FORM_SUCCESS:
      let updatedUser: User = (<UserActionUnion>action).payload;
      let usersListCopy2: User[] = [...state.usersList];
      usersListCopy2 = usersListCopy2.map((user) =>
        user.id == updatedUser.id ? updatedUser : user
      );

      return {
        ...state,
        usersList: usersListCopy2,
        msgState: MsgState.UPDATED,
      };

    case UserActionType.UPDATE_USER_FORM_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    // delete a user actions
    case UserActionType.DELETE_A_USER:
      return {
        ...state,
        msgState: MsgState.LOADING,
      };

    case UserActionType.DELETE_A_USER_SUCCESS:
      let deletedUser: User = (<UserActionUnion>action).payload;
      let deletedUserIndex = state.usersList.indexOf(deletedUser);
      let usersListCopy3: User[] = [...state.usersList];
      usersListCopy3.splice(deletedUserIndex, 1);
      return {
        ...state,
        msgState: MsgState.LOADED,
        usersList: usersListCopy3,
      };

    case UserActionType.DELETE_A_USER_ERROR:
      return {
        ...state,
        msgState: MsgState.ERR,
        userServerErrorMsg: (<UserActionUnion>action).payload,
      };

    default:
      return { ...state };
  }
}
